@@ -152,6 +152,9 @@ typename MatType::elem_type ActiveCMAES<SelectionPolicyType,
   // The current visitation order (sorted by population objectives).
   UVecType idx = linspace<UVecType>(0, lambda - 1, lambda);
 
+  const size_t actualMaxIterations = (maxIterations == 0) ?
+      std::numeric_limits<size_t>::max() : maxIterations;
+
   // Now iterate!
   Callback::BeginOptimization(*this, function, transformedIterate,
       callbacks...);
@@ -163,11 +166,11 @@ typename MatType::elem_type ActiveCMAES<SelectionPolicyType,
   size_t patience = 10 + (30 * iterate.n_elem / lambda) + 1;
   size_t steps = 0;
 
-  for (size_t i = 1; (i != maxIterations) && !terminate; ++i)
+  for (size_t i = 0; i < actualMaxIterations && !terminate; ++i)
   {
     // To keep track of where we are.
-    idx0 = (i - 1) % 2;
-    idx1 = i % 2;
+    idx0 = i % 2;
+    idx1 = (i + 1) % 2;
 
     // Perform Cholesky decomposition. If the matrix is not positive definite,
     // add a small value and try again.
