@@ -57,8 +57,8 @@ template<typename SeparableFunctionType,
          typename MatType,
          typename GradType,
          typename... CallbackTypes>
-typename std::enable_if<IsArmaType<GradType>::value,
-typename MatType::elem_type>::type
+typename std::enable_if<IsMatrixType<GradType>::value,
+    typename MatType::elem_type>::type
 SnapshotSGDR<UpdatePolicyType>::Optimize(
     SeparableFunctionType& function,
     MatType& iterate,
