@@ -9,37 +9,30 @@
  * the 3-clause BSD license along with ensmallen.  If not, see
  * http://www.opensource.org/licenses/BSD-3-Clause for more information.
  */
-
+#if defined(ENS_USE_COOT)
+  #include <armadillo>
+  #include <bandicoot>
+#endif
 #include <ensmallen.hpp>
 #include "catch.hpp"
 #include "test_function_tools.hpp"
+#include "test_types.hpp"
 
 using namespace ens;
 using namespace ens::test;
 
-/**
- * Run IQN on logistic regression and make sure the results are acceptable.
- */
-TEST_CASE("IQNLogisticRegressionTest", "[IQNTest]")
-{
-  // Run on a couple of batch sizes.
-  for (size_t batchSize = 1; batchSize < 9; batchSize += 4)
-  {
-    IQN iqn(0.01, batchSize, 5000, 0.01);
-    LogisticRegressionFunctionTest(iqn, 0.013, 0.016);
-  }
-}
+// NOTE: IQN cannot use arma::hmat because pinv() is required.
 
-/**
- * Run IQN on logistic regression and make sure the results are acceptable.  Use
- * arma::fmat.
- */
-TEST_CASE("IQNLogisticRegressionFMatTest", "[IQNTest]")
+TEMPLATE_TEST_CASE("IQN_LogisticRegressionFunction", "[IQN]", ENS_TEST_TYPES)
 {
   // Run on a couple of batch sizes.
   for (size_t batchSize = 1; batchSize < 9; batchSize += 4)
   {
-    IQN iqn(0.001, batchSize, 5000, 0.01);
-    LogisticRegressionFunctionTest<arma::fmat>(iqn, 0.013, 0.016);
+    IQN iqn(0.01, batchSize, 200000, 0.01);
+    // It could take a few attempts to converge.
+    LogisticRegressionFunctionTest<TestType>(iqn,
+        Tolerances<TestType>::LRTrainAcc,
+        Tolerances<TestType>::LRTestAcc,
+        5);
   }
 }
